@@ -53,12 +53,9 @@
 //! [1]
 RasterWindow::RasterWindow(QWindow *parent)
     : QWindow(parent)
+    , m_backingStore(new QBackingStore(this))
 {
-    create();
-    m_backingStore = new QBackingStore(this);
-
     setGeometry(100, 100, 300, 200);
-
 }
 //! [1]
 
@@ -114,6 +111,7 @@ void RasterWindow::renderNow()
 
     painter.fillRect(0, 0, width(), height(), Qt::white);
     render(&painter);
+    painter.end();
 
     m_backingStore->endPaint();
     m_backingStore->flush(rect);
